#Hier findet man alle Funktionen zur: Bewegungserkennung, Messung der Temperatur und Feuchtigkeit

import RPi.GPIO as GPIO
import cv2, picamera, io, time, copy, sys, Adafruit_DHT,subprocess
import numpy as np


#Bewegungserkennung

#Funktion fuer differential images
def diffImg(t0, t1, t2):
	d1=cv2.absdiff(t2,t1)
	d2=cv2.absdiff(t1,t0)
	return cv2.bitwise_and(d1,d2)


#Bewegungserkennung braucht eine Initialisierung, damit es die ersten
#drei Bilder macht- man braucht wenigstens drei Bilder um Beweung 
#zu erkennen. Spaeter macht es immer nur ein Foto

#Funktion fuer Initalisierung von Motion Detection
#Variblen: Array[3]-pictures, float time

def motDetInit(pictures,t):
	try:
		with picamera.PiCamera() as camera:
			print("Checking for Movement")
			picturesGray=[0,0,0]
			for i in range(0,3):
				#Foto Machen
				stream=io.BytesIO()
				print("capturing Photo")
				camera.capture(stream, format='jpeg')
				print("Photo captured")
				data= np.fromstring(stream.getvalue(), dtype=np.uint8)
				pictures[i]=cv2.imdecode(data,1)
				picturesGray[i]=cv2.cvtColor(pictures[i],cv2.COLOR_RGB2GRAY)
				stream.close()
				time.sleep(t)
			
			result=diffImg(picturesGray[0],picturesGray[1],picturesGray[2])
			#Bild modifizieren so, dass es nur weiss und schwarz ist 
			s,image=cv2.threshold(result,15,256,cv2.THRESH_BINARY)
			#weisse Pixeln zaehlen-Aenderungsmass
			white=cv2.countNonZero(image)
			print("Number of changed pixels is ")
			print(white)
			
			##Wenn es zu viel Aenderung gibt: Alarm

			if white>4000:
				print("Movement detected!")
				#Konturen von der Bewegung werden gefunden und gezeichnet
				a,b=cv2.findContours(image, cv2.RETR_LIST,cv2.CHAIN_APPROX_SIMPLE)
				overlay=copy.copy(pictures[1])
				cv2.drawContours(overlay,a,-1,(0,0,255),3)
				moved=cv2.addWeighted(overlay,0.3,pictures[1],0.7,0.0)
				return (True, moved)
			else:
				print("No Movement Detected")
				return (False, 0)
	except KeyboardInterrupt:
		print("KeyboardInterrupt in motDetInit")
		sys.exit()

#Weiter machen
def motDetCont(pictures, t):
	try:
		with picamera.PiCamera() as camera:
			print("Checking for Movement")
			#Foto Machen
			stream=io.BytesIO()
			print("capturing Photo")
			camera.capture(stream, format='jpeg')
			print("Photo captured")
			data= np.fromstring(stream.getvalue(), dtype=np.uint8)
			temp=cv2.imdecode(data,1)
			tempGray=cv2.cvtColor(temp,cv2.COLOR_RGB2GRAY)
			stream.close()
			time.sleep(t)

			pictures[0]=pictures[1]
			pictures[1]=pictures[2]
			pictures[2]=temp

			tempGray1=cv2.cvtColor(pictures[0],cv2.COLOR_RGB2GRAY)
			tempGray2=cv2.cvtColor(pictures[1],cv2.COLOR_RGB2GRAY)
			
			result=diffImg(tempGray1,tempGray2,tempGray)
			s,image=cv2.threshold(result,15,256,cv2.THRESH_BINARY)
			white=cv2.countNonZero(image)
			print("Number of changed pixels is ")
			print(white)
		
			if white>4000:
				print("Movement detected!")
				#Konturen von der Bewegung werden gefunden und gezeichnet
				a,b=cv2.findContours(image, cv2.RETR_LIST,cv2.CHAIN_APPROX_SIMPLE)
				overlay=copy.copy(pictures[1])
				cv2.drawContours(overlay,a,-1,(0,0,255),3)
				moved=cv2.addWeighted(overlay,0.3,pictures[1],0.7,0.0)
				return (True,moved)
			else:
				print("No Movement Detected")
				return (False, 0)
	except KeyboardInterrupt:
		print("KeyboardInterrupt in motDetInit")
		sys.exit()






#Funktionen fuer Messung der Temperatur und Feuchtigkeit

#CPU Temperatur
def get_temperature():
    s = subprocess.check_output(['/opt/vc/bin/vcgencmd','measure_temp']).decode('utf-8')
    return float(s.split('=')[1][:-3])

#Temperatur gemessen am gewoehnlichen Thermometer-sensor
def get_sensor_temp(number):
        try:
		if number==1:
			t1=open("/sys/bus/w1/devices/28-03146820f5ff/w1_slave")
        	        text_1=t1.read()
        	        t1.close()
        	        s1=text_1.split("\n")[1]
        	        tdata1=s1.split(" ")[9]
        	        temperature_1=float(tdata1[2:])
        	        temperature_1=temperature_1/1000
        	        return temperature_1
        	elif number==2:
        	        t2=open("/sys/bus/w1/devices/28-0314688813ff/w1_slave")
        	        text_2=t2.read()
        	        t2.close()
        	        s2=text_2.split("\n")[1]
        	        tdata2=s2.split(" ")[9]
        	        temperature_2=float(tdata2[2:])
        	        temperature_2=temperature_2/1000
        	        return temperature_2
	except KeyboardInterrupt:
		print("KeyboardInterrupt in motDetInit")
		sys.exit()

#Feuchtigkeitsmessung
def get_hygrometer_humidity(number):
	try:
		if number==1:
			sensor = Adafruit_DHT.DHT22
			pin = 17
			humidity, temperature = Adafruit_DHT.read_retry(sensor, pin)
			return humidity
		if number==2:
			sensor = Adafruit_DHT.DHT22
			pin = 27
			humidity, temperature = Adafruit_DHT.read_retry(sensor, pin)
			return humidity
	except KeyboardInterrupt:
		print("KeyboardInterrupt in motDetInit")
		sys.exit()

#Temperatur gemessen an Hygrometer
def get_hygrometer_temp(number):
	try:
		if number==1:
			sensor = Adafruit_DHT.DHT22
			pin = 17
			humidity, temperature = Adafruit_DHT.read_retry(sensor, pin)
			return temperature
		if number==2:
			sensor = Adafruit_DHT.DHT22
			pin = 27
			humidity, temperature = Adafruit_DHT.read_retry(sensor, pin)
			return temperature
	except KeyboardInterrupt:
		print("KeyboardInterrupt in motDetInit")
		sys.exit()



#der Interface fuer ANALOG To DIGITAL CONVERTER (ADC)// d.h. fuer dem RAUCHMALEDER 

GPIO.setmode(GPIO.BOARD)

PIN_CLK=35
PIN_DI =33
PIN_DO =15
PIN_CS =13

GPIO.setup(PIN_DI, GPIO.OUT)
GPIO.setup(PIN_DO, GPIO.IN)
GPIO.setup(PIN_CLK,GPIO.OUT)
GPIO.setup(PIN_CS, GPIO.OUT)

def getADC(channel):
	GPIO.output(PIN_CS, True)	#clear last transmission
	GPIO.output(PIN_CS, False)

	#start clock
	GPIO.output(PIN_CLK, False)
	
	#Input MUX address
	for i in [1,1,channel]:#start bit + mux assignement
		if (i==1):
			GPIO.output(PIN_DI, True)
		else:
			GPIO.output(PIN_DI, False)

		GPIO.output(PIN_CLK, True)
		GPIO.output(PIN_CLK, False)

	# Read 8 ADC bits
	ad=0
	for i in range(8):
		GPIO.output(PIN_CLK, True)
		GPIO.output(PIN_CLK, False)
		ad<<= 1 #bit shift
		if (GPIO.input(PIN_DO)):
			ad|=0x1 # set first bit

	#reset
		GPIO.output(PIN_CS, True)

		return ad
